#include <gtk/gtk.h>
#include <string.h>
#include "parse_data.h"
#include "conf_file.h"
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <zip.h>

gboolean on_menu_configuration(GtkMenuItem *menuitem,gpointer user_data);

/***********************************************************************************************

	setup_configuration_command_line

return a pointer to the command line for adding the biosset and ramoption parameter
WARNING : the return pointer must be freed by the caller !

***********************************************************************************************/

char * setup_configuration_command_line(char * machine_name,conf_t * conf)
{
	const char * conf_bios = NULL;
	const char * conf_ram = NULL;
	char * command = NULL;

	conf_bios = conf_get(conf,machine_name,BIOS_TYPE);	
	conf_ram = conf_get(conf,machine_name,RAM_TYPE);	

	if( conf_bios == NULL && conf_ram == NULL ) return NULL;

	command = malloc(1);
	command[0]=0;

	if(conf_bios != NULL) {
		command = realloc(command,sizeof(command)+strlen(conf_bios)+sizeof(" -bios ")+sizeof(" ")+1); /* +1 for terminating 0 */
		strcat(command," -bios ");
		strcat(command,conf_bios);
		strcat(command," ");
	}
	if(conf_ram != NULL) {
		command = realloc(command,sizeof(command)+strlen(conf_ram)+sizeof(" -ramsize ")+sizeof(" ")+1); /* +1 for terminating 0 */
		strcat(command," -ramsize ");
		strcat(command,conf_ram);
		strcat(command," ");
	}

	return command;
}

void FillConfigurationList(char * machine_name,GtkWidget * ConfigurationTree, llist_t * mess, conf_t * conf)
{
	GtkListStore * ConfigurationListStore;
	GtkTreeIter iter;
	const char * conf_bios = NULL;
	const char * conf_ram = NULL;
	llist_t * machine;

	ConfigurationListStore=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ConfigurationTree)));

	conf_bios = conf_get(conf,machine_name,BIOS_TYPE);	
	conf_ram = conf_get(conf,machine_name,RAM_TYPE);	

	/* look for the right machine */
	machine = find_node_by_attr(mess,"machine","name",machine_name);

	if ( machine == NULL ) return;

	gtk_list_store_clear(ConfigurationListStore);

	/* if no biosset was already selected, choose thedefault one */
	llist_t * biosset;
	if( conf_bios == NULL) {
		biosset = find_node_by_attr(machine,"biosset","default","yes");
	}
	else {
		biosset = find_node_by_attr(machine,"biosset","name",conf_bios);
	}

	if( biosset) {
		gtk_list_store_append (ConfigurationListStore, &iter);
		gtk_list_store_set (ConfigurationListStore, &iter,
				0, BIOS_TYPE,
				1, find_attr(biosset,"name"),
				2, find_attr(biosset,"description"),
				-1);
		conf_set(conf,machine_name,BIOS_TYPE,find_attr(biosset,"name"),0);
	}

	/* display default ramoption if none already selected */
	llist_t * ramoption;
	if( conf_ram == NULL) {
		ramoption = find_node_by_attr(machine,"ramoption","default","1");
		if( ramoption ) {
			gtk_list_store_append (ConfigurationListStore, &iter);
			gtk_list_store_set (ConfigurationListStore, &iter,
					0, RAM_TYPE,
					1, get_data(ramoption),
					-1);
			conf_set(conf,machine_name,RAM_TYPE,get_data(ramoption),0);
		}
	}
	/* display the selected ramoption */
	else {
		gtk_list_store_append (ConfigurationListStore, &iter);
		gtk_list_store_set (ConfigurationListStore, &iter,
				0, RAM_TYPE,
				1, conf_ram,
				-1);
	}
}





void UpdateConfigurationList(char * machine_name,GtkWidget * ConfigurationTree, llist_t * mess, conf_t * conf)
{
	GtkListStore * ConfigurationListStore;
	GtkTreeModel*   model;
	GtkTreeIter iter;
	gchar * ConfType;
	gchar * ConfValue;
	GtkTreeSelection *sel;
	llist_t * machine;

	/* get selected configuration */
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(ConfigurationTree));
	if( ! gtk_tree_selection_count_selected_rows( sel ) ) /* if no selection */
		return;
	gtk_tree_selection_get_selected(sel,&model,&iter);
	gtk_tree_model_get (model,&iter,0,&ConfType,1,&ConfValue, -1);

	ConfigurationListStore=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ConfigurationTree)));

	/* look for the right machine */
	machine = find_node_by_attr(mess,"machine","name",machine_name);

	if( machine == NULL ) return;

	/* next BIOS on the list */
	if(strcmp(ConfType,BIOS_TYPE)==0) {
		llist_t * biosset;
		biosset = find_node_by_attr(machine,"biosset","name",ConfValue);
		if( biosset != NULL ) {
			/* display next biosset */
			if( find_next_node(biosset) != NULL ) {
				biosset = find_next_node(biosset);
			}
			else {
				biosset = find_first_node(machine,"biosset");
			}

			if( biosset != NULL ) {
				gtk_list_store_set (ConfigurationListStore, &iter,
						0, BIOS_TYPE,
						1, find_attr(biosset,"name"),
						2, find_attr(biosset,"description"),
						-1);
				conf_set(conf,machine_name,BIOS_TYPE,find_attr(biosset,"name"),0);
			}
		}
	}

	/* next RAM on the list */
	if(strcmp(ConfType,RAM_TYPE)==0) {
		
		llist_t * ramoption;
		/* display next ramoption if any */
		ramoption = find_node_by_attr(machine,"ramoption","name",ConfValue);
		if( ramoption ) {
			if( find_next_node(ramoption) != NULL ) {
				ramoption = find_next_node(ramoption);
			}
			else {
				ramoption = find_first_node(machine,"ramoption");
			}

			if( ramoption ) {
				gtk_list_store_set (ConfigurationListStore, &iter,
						0, RAM_TYPE,
						1, get_data(ramoption),
						-1);
				conf_set(conf,machine_name,RAM_TYPE,get_data(ramoption),0);
			}
		}
	}

	g_free(ConfType);
	g_free(ConfValue);
}



void AddConfigurationMenuItem(char * machine_name,GtkWidget * ConfigurationTree,GtkWidget * menu, llist_t * mess, conf_t * conf)
{
	GtkListStore * ConfigurationListStore;
	GtkTreeModel*   model;
	GtkTreeIter iter;
	gchar * ConfType;
	gchar * ConfValue;
	GtkTreeSelection *sel;
	GtkWidget * menu_item;
	llist_t * machine;

	/* get selected configuration */
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(ConfigurationTree));
	if( ! gtk_tree_selection_count_selected_rows( sel ) ) /* if no selection */
		return;
	gtk_tree_selection_get_selected(sel,&model,&iter);
	gtk_tree_model_get (model,&iter,0,&ConfType,1,&ConfValue, -1);

	ConfigurationListStore=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ConfigurationTree)));

	/* look for the right machine */
	machine = find_node_by_attr(mess,"machine","name",machine_name);

	if( machine == NULL ) return;

	/* next BIOS on the list */
	if(strcmp(ConfType,BIOS_TYPE)==0) {
		llist_t * biosset;
		biosset = find_first_node(machine,"biosset");
		/* display next biosset */
		while ( biosset != NULL )  {
			menu_item=gtk_menu_item_new_with_label (find_attr(biosset,"description"));
			gtk_widget_show(menu_item);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
			g_signal_connect(GTK_OBJECT(menu_item), "button-release-event",
					G_CALLBACK(on_menu_configuration), biosset);
			biosset=find_next_node(biosset);
		}
	}

	/* next RAM on the list */
	if(strcmp(ConfType,RAM_TYPE)==0) {
		llist_t * ramoption;
		ramoption = find_first_node(machine,"ramoption");

		/* display next ramoption if any */
		while ( ramoption != NULL )  {
			menu_item=gtk_menu_item_new_with_label (get_data(ramoption));
			gtk_widget_show(menu_item);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
			g_signal_connect(GTK_OBJECT(menu_item), "button-release-event",
					G_CALLBACK(on_menu_configuration), ramoption);
			ramoption=find_next_node(ramoption);
		}
	}

	g_free(ConfType);
	g_free(ConfValue);
}

