#include <gtk/gtk.h>
#include <strings.h>
#include <string.h>
#include <unistd.h>
#include "machine_list.h"
#include "device_list.h"
#include "software_list.h"
#include "configuration_list.h"
#include "conf_file.h"
#include "../config.h"
#include "linked_list.h"


static llist_t *		mess;
/* Main window widget */
static GtkWidget *		MainWindow;
static GtkWidget *		MachineTree;
static GtkWidget *		DeviceTree;
static GtkWidget *		SoftwareTree;
static GtkWidget *		ConfigurationTree;
static GtkWidget *		StatusBar;
static GtkWidget *		ToggleFullScreen;
static GtkWidget *		ToggleMaximize;
static GtkWidget *		ToggleSound;
static GtkWidget *		ToggleRenameSnap;
static GtkWidget *		ToggleSeeParentClonesSoftware;
static GtkWidget *		MachineImageContainer;
static GtkWidget *		SoftwareImageContainer;
static GtkWidget *		hpaned_machine;
static GtkWidget *		vpaned_machine;
static GtkWidget *		hpaned_software;
static GtkWidget *		vpaned_software_image;
static GtkWidget *		vpaned_device;
/* Path window widget */
static GtkWidget *		PathWindow;
static GtkWidget *		PathCancel;
static GtkWidget *		PathMessEntry;
static GtkWidget *		PathSoftwareList;
static GtkWidget *		PathMachineSnapList;
static GtkWidget *		PathSoftwareSnapList;

static char *			SelectedMachine=NULL;
static int			SoftwareAlreadyResized=0;
static int			MachineAlreadyResized=0;
static int			enable_parent_clones = 0;

static conf_t			gnomess_conf;
static int			conf_loaded = 0;
static int			conf_done = 0; /* Let the number of machine appears at start in the status bar instead of being erased by machine status */
static char			additional_parameters[1024];

void str_tolower(char * string);

/***************************************************************************


  get_snap_software_name

WARNING, the returned pointer must be freed

Retrieve the software with "snap" notification
 ***************************************************************************/
char * get_snap_software_name()
{
        GtkTreeModel*   model;
        GtkTreeIter iter;
        gchar * software;
	int snap;

	/* search the "snap" button */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree));
	if( gtk_tree_model_get_iter_first (model,&iter) == FALSE ) return NULL;
	do {
		gtk_tree_model_get (model,&iter,3,&software,4,&snap, -1);
		if(snap) return software;
		g_free(software);
	}
	while( gtk_tree_model_iter_next(model,&iter) != FALSE );

	return NULL;
}

/***************************************************************************


  SoftwareShortName 


 ***************************************************************************/
static const char * SoftwareShortName(const char * software)
{
	int i = strlen(software);
	while(i>=0) {
		if(software[i]=='/') {
			return (software+i+1);
		}
		i--;
	}
	return software;
}
/***************************************************************************


  FindDeviceFromExtension 


 ***************************************************************************/
static char * FindDeviceFromExtension(char * machine_name,char * extension_name)
{
	llist_t * machine;
	llist_t * device;
	llist_t * extension;
	llist_t * instance;

	if(extension_name == NULL) return NULL;
	if(machine_name == NULL) return NULL;

	machine = find_node_by_attr(mess,"machine","name",machine_name);
	if( machine == NULL ) return NULL;

	device = find_first_node(machine,"device");
	while( device != NULL )  {
		instance = find_first_node(device,"instance");
		if( instance == NULL ) return NULL;
		extension = find_first_node(device,"extension");
		while( extension != NULL ) {
			if(strcmp( find_attr(extension,"name") , extension_name) == 0 ) {
				return find_attr(instance,"name");
			}
			extension=find_next_node(extension);
		}
		device=find_next_node(device);
	};

	return NULL;
}
/***************************************************************************


  MountSoftware 


 ***************************************************************************/
int MountSoftware(const char * device,const  char * software)
{
	gchar *	device_name;
	gchar *	software_name;
	gchar *	software_extension;
	char *	extension_device;
	GtkTreeModel*	model;
	GtkTreeIter	iter;
	GtkTreeSelection *sel;
	int found = 0;
	int path_int = 0 ;

	/* auto-mount, get the selected machine + software and find the
	   first matching device */
	if(device == NULL || software == NULL) {

		/* get selected software */
		sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(SoftwareTree));
		/* if no selection */
		if( ! gtk_tree_selection_count_selected_rows( sel ) )
			return FALSE;
		gtk_tree_selection_get_selected(sel,&model,&iter);
		gtk_tree_model_get (model,&iter,2,&software_extension,1,&software_name, -1);

		if( (extension_device = FindDeviceFromExtension(SelectedMachine,software_extension)) == NULL)
			return FALSE;

		/* Update Device list */
		model = gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree));
		if( gtk_tree_model_get_iter_first (model,&iter) == FALSE ) return FALSE;
		do {
			gtk_tree_model_get (model,&iter,1,&device_name, -1);
			if(strcmp(device_name,extension_device)==0) {
				gtk_list_store_set (GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree))), &iter,2, software_name,3,SoftwareShortName(software_name),-1);
				conf_set(&gnomess_conf,SelectedMachine,device_name,software_name,FALSE);
			}
			g_free(device_name);
		}
		while( gtk_tree_model_iter_next(model,&iter) != FALSE );

		g_free(software_extension);
		g_free(software_name);

		return TRUE;
	}

	/* normal mount of the specified software on the specified device */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree));
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(DeviceTree));
	if( gtk_tree_model_get_iter_first (model,&iter) == FALSE ) return FALSE;
	do {
		gtk_tree_model_get (model,&iter,1,&device_name, -1);
		if(strcmp(device_name,device)==0) {
			found = 1;
			break;
		}
		path_int++;
		g_free(device_name);
	}
	while( gtk_tree_model_iter_next(model,&iter) != FALSE );

	if(found==1) {

		if( !gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree))),&iter,NULL,path_int)) return FALSE;
		gtk_list_store_set (GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree))), &iter,2, software,3,SoftwareShortName(software),-1);

		conf_set(&gnomess_conf,SelectedMachine,device,software,FALSE);
		return TRUE;
	}

	return FALSE;
}

/***************************************************************************


  MountSoftwareInConfFile


 ***************************************************************************/
void MountSoftwareInConfFile(const char * machine_name,conf_t * conf, llist_t * mess)
{
	const char * software;
	llist_t * machine;
	llist_t * device;
	llist_t * instance;

	if(machine_name == NULL) return;

	machine = find_node_by_attr(mess,"machine","name",machine_name);

	if( machine == NULL ) return;

	device = find_first_node(machine,"device");
	while ( device != NULL ) {
		instance = find_first_node(device,"instance");
		while ( instance != NULL ) {
			software = conf_get(conf, machine_name, find_attr(instance,"name"));
			if( software )
				MountSoftware(find_attr(instance,"name"),software);
			instance=find_next_node(instance);
		} 
		device=find_next_node(device);
	}
}
/***************************************************************************


  select_machine 


 ***************************************************************************/
gboolean select_machine_cb(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data)
{
	gchar * machine_name;
	const char * machine = (const char *)data;
	GtkTreeSelection *sel;

	gtk_tree_model_get (model,iter,0,&machine_name, -1);
	if(strcmp(machine_name,machine)==0) {
		gtk_tree_view_expand_to_path( GTK_TREE_VIEW( MachineTree ), path );
		sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(MachineTree));
		gtk_tree_selection_select_path (sel, path); 
		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW( MachineTree ),path,NULL,FALSE,0.0,0.0);
		gtk_tree_view_set_cursor( GTK_TREE_VIEW( MachineTree ), path, NULL, FALSE ); /* launch on_cursor_changed */
		g_free(machine_name);
		return TRUE;;
	}
	g_free(machine_name);
	return FALSE;
}
static void select_machine(const char * machine)
{
	GtkTreeModel*   model;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(MachineTree));

	gtk_tree_model_foreach(model,select_machine_cb,(gpointer)machine);
}

/***************************************************************************


  run_mess 


 ***************************************************************************/
static void run_mess(int run_bios_only)
{
	char command_line[10000];
	gchar *	software_name;
	gchar *	device_name;
	char *	configuration_option;
	GtkTreeModel*	model;
	GtkTreeIter	iter;
	FILE *fpipe;
	int return_value;
	guint id;
	char tmp[1024];

	sprintf(command_line,"%s %s ",conf_get(&gnomess_conf,"mess","path"),
						additional_parameters);

	/* add configuration option */
	configuration_option = setup_configuration_command_line(SelectedMachine,&gnomess_conf);
	if( configuration_option != NULL) {
		strcat(command_line,configuration_option);
		free(configuration_option);
	}
	/* add driver name */
	strcat(command_line,SelectedMachine);
	strcat(command_line," ");

	/* Get mounted software */
	if( ! run_bios_only ) {
		model = gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree));
		if( gtk_tree_model_get_iter_first (model,&iter) == FALSE ) return;
		do {
			gtk_tree_model_get (model,&iter,1,&device_name,2,&software_name, -1);
			if( software_name != NULL ) {
				strcat(command_line,"-");
				strcat(command_line,device_name);
				strcat(command_line," \"");
				strcat(command_line,software_name);
				strcat(command_line,"\" ");
			}
			g_free(device_name);
			g_free(software_name);
		}
		while( gtk_tree_model_iter_next(model,&iter) != FALSE );
	}

	/* Be sure to read both standard and error MESS output */
	strcat(command_line," 2>&1 ");

	/* Time stamp for snap shots */
	time_t start_time;
	time(&start_time);

	printf("gnomess launching command : %s\n",command_line);

	char * line = NULL;
	char * output_buf = NULL;
	line = malloc(65535);
	output_buf = malloc(1);
	output_buf[0] = 0;
	if( line == NULL || output_buf == NULL ) {
		printf("Not enough memory for output buffer\n");
		return;
	}

	fpipe = (FILE*)popen(command_line,"r");
	if(fpipe==NULL) return;

        while ( fgets( line, sizeof line, fpipe)) {
		output_buf = realloc(output_buf, strlen(output_buf) + strlen(line) + 1);
		strcat(output_buf, line);
        }
	return_value=pclose(fpipe);

	/* rename the snapshot to the "snap" software name */
	if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(ToggleRenameSnap))) {
		char * snap_software = get_snap_software_name();
		if( snap_software != NULL ) {
			RenameSnapshot(SelectedMachine,snap_software,start_time,&gnomess_conf);
			ShowSoftwareImage(GTK_CONTAINER(SoftwareImageContainer),SelectedMachine,mess,snap_software,&gnomess_conf,enable_parent_clones);
			free(snap_software);
		}
	}

	/* Message box if error */
	if( return_value != 0 ) {
		GtkWidget * dialog;
		dialog = gtk_message_dialog_new (GTK_WINDOW(MainWindow),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_ERROR,
				GTK_BUTTONS_CLOSE,
				"MESS reports error :");
		gtk_message_dialog_format_secondary_text ( GTK_MESSAGE_DIALOG(dialog), "%s", output_buf );
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
	}
	free(line);
	free(output_buf);

	/* Update status bar */	
	sprintf(tmp,"MESS returns code : %d",return_value);

	id=gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar),"general");
	gtk_statusbar_pop(GTK_STATUSBAR(StatusBar),id);
	gtk_statusbar_push(GTK_STATUSBAR(StatusBar),id,tmp);

}
/***************************************************************************


  conf_start 

Read the configuration file and set up the data
 ***************************************************************************/
static void conf_start(conf_t *conf)
{
	const char *dummy;
	int width = 600;
	int height = 400;

	/* gnomess */


	/* MainWindow size */
	dummy = conf_get(conf,"gnomess","main_width");
	if(dummy!=NULL) {
		width = atoi(dummy);
	}
	dummy = conf_get(conf,"gnomess","main_height");
	if(dummy!=NULL) {
		height = atoi(dummy);
	}
	gtk_window_resize(GTK_WINDOW(MainWindow),width, height);

	/* Machine hpane */
	dummy = conf_get(conf,"gnomess","machine_hpane");
	if(dummy!=NULL) {
		gtk_paned_set_position(GTK_PANED(hpaned_machine),atoi(dummy));
	}
	/* Machine vpane */
	dummy = conf_get(conf,"gnomess","machine_vpane");
	if(dummy!=NULL) {
		gtk_paned_set_position(GTK_PANED(vpaned_machine),atoi(dummy));
	}
	/* Software hpane */
	dummy = conf_get(conf,"gnomess","software_hpane");
	if(dummy!=NULL) {
		gtk_paned_set_position(GTK_PANED(hpaned_software),atoi(dummy));
	}
	/* Software image vpane */
	dummy = conf_get(conf,"gnomess","software_image_vpane");
	if(dummy!=NULL) {
		gtk_paned_set_position(GTK_PANED(vpaned_software_image),atoi(dummy));
	}
	/* Device vpane */
	dummy = conf_get(conf,"gnomess","device_vpane");
	if(dummy!=NULL) {
		gtk_paned_set_position(GTK_PANED(vpaned_device),atoi(dummy));
	}
	/*Fullscreen*/
	dummy = conf_get(conf,"gnomess","fullscreen");
	if(dummy!=NULL && strcmp(dummy,"yes")==0)
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleFullScreen),TRUE);
	else if(dummy!=NULL && strcmp(dummy,"no")==0)
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleFullScreen),FALSE);
	else {
		conf_set(conf,"gnomess","fullscreen","no",FALSE);
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleFullScreen),FALSE);
	}
	/*Maximize*/
	dummy = conf_get(conf,"gnomess","maximize");
	if(dummy!=NULL && strcmp(dummy,"yes")==0)
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleMaximize),TRUE);
	else if(dummy!=NULL && strcmp(dummy,"no")==0)
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleMaximize),FALSE);
	else {
		conf_set(conf,"gnomess","maximize","no",FALSE);
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleMaximize),FALSE);
	}
	/*Sound*/
	dummy = conf_get(conf,"gnomess","sound");
	if(dummy!=NULL && strcmp(dummy,"yes")==0)
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleSound),TRUE);
	else if(dummy!=NULL && strcmp(dummy,"no")==0)
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleSound),FALSE);
	else {
		conf_set(conf,"gnomess","sound","yes",FALSE);
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleSound),TRUE);
	}
	/*RenameSnap*/
	dummy = conf_get(conf,"gnomess","rename_snap");
	if(dummy!=NULL && strcmp(dummy,"yes")==0)
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleRenameSnap),TRUE);
	else if(dummy!=NULL && strcmp(dummy,"no")==0)
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleRenameSnap),FALSE);
	else {
		conf_set(conf,"gnomess","rename_snap","no",FALSE);
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleRenameSnap),FALSE);
	}

	/* Selected Machine */
	dummy = conf_get(conf,"gnomess","selected_machine");
	if(dummy!=NULL) {
		select_machine(dummy);
		/* Selected Software */
		MountSoftwareInConfFile(dummy,&gnomess_conf,mess);
		/* Set snap selection on device*/
		SetSnapSelection((char *)dummy,DeviceTree,conf);
	}

	conf_loaded = 1;
}

/***************************************************************************


  update_additional_parameters 

take care of the data in gnomess_conf and update the aditional parameters on the command line used to launch MESS
 ***************************************************************************/
static void update_additional_parameters(conf_t *conf)
{
	if(!conf_loaded) return;

	bzero(additional_parameters,1024);

	/*Fullscreen*/
	if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(ToggleFullScreen))) {
		strcat(additional_parameters," -nowindow ");
		conf_set(conf,"gnomess","fullscreen","yes",FALSE);
	}
	else {
		strcat(additional_parameters," -window ");
		conf_set(conf,"gnomess","fullscreen","no",FALSE);
	}
	/*Maximize*/
	if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(ToggleMaximize))) {
		strcat(additional_parameters," -maximize ");
		conf_set(conf,"gnomess","maximize","yes",FALSE);
	}
	else {
		strcat(additional_parameters," -nomaximize ");
		conf_set(conf,"gnomess","maximize","no",FALSE);
	}
	/*Sound*/
	if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(ToggleSound))) {
		strcat(additional_parameters," -sound ");
		conf_set(conf,"gnomess","sound","yes",FALSE);
		gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(ToggleSound),"gnomess_sound");
	}
	else {
		strcat(additional_parameters," -nosound ");
		conf_set(conf,"gnomess","sound","no",FALSE);
		gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(ToggleSound),"gnomess_nosound");
	}
	/*RenameSnap*/
	if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(ToggleRenameSnap))) {
		conf_set(conf,"gnomess","rename_snap","yes",FALSE);
	}
	else {
		conf_set(conf,"gnomess","rename_snap","no",FALSE);
	}
}

/***************************************************************************


  SetToggleSeeParentClonesSoftware 


 ***************************************************************************/
void SetToggleSeeParentClonesSoftware(void)
{
	const char *dummy = conf_get(&gnomess_conf,SelectedMachine,"SeeParentClonesSoftware");
	if(dummy!=NULL && strcmp(dummy,"yes")==0) {
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleSeeParentClonesSoftware),TRUE);
		enable_parent_clones = 1;
	}
	else if(dummy!=NULL && strcmp(dummy,"no")==0) {
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleSeeParentClonesSoftware),FALSE);
		enable_parent_clones = 0;
	}
	else {
		conf_set(&gnomess_conf,SelectedMachine,"SeeParentClonesSoftware","no",FALSE);
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ToggleSeeParentClonesSoftware),FALSE);
		enable_parent_clones = 0;
	}

}

/***************************************************************************


  on_ToggleRenameSnap_toggled 


 ***************************************************************************/
void on_ToggleRenameSnap_toggled(GtkToggleToolButton *toggle_tool_button, gpointer user_data)
{
	update_additional_parameters(&gnomess_conf);
}
/***************************************************************************


  on_ToggleSound_toggled 


 ***************************************************************************/
void on_ToggleSound_toggled(GtkToggleToolButton *toggle_tool_button, gpointer user_data)
{
	update_additional_parameters(&gnomess_conf);
}
/***************************************************************************


  on_ToggleMaximize_toggled 


 ***************************************************************************/
void on_ToggleMaximize_toggled(GtkToggleToolButton *toggle_tool_button, gpointer user_data)
{
	update_additional_parameters(&gnomess_conf);
}
/***************************************************************************


  on_ToggleFullScreen_toggled 


 ***************************************************************************/
void on_ToggleFullScreen_toggled(GtkToggleToolButton *toggle_tool_button, gpointer user_data)
{
	update_additional_parameters(&gnomess_conf);
}
/***************************************************************************


  on_SeeParentClonesSoftware_toggled 


 ***************************************************************************/
void on_SeeParentClonesSoftware_toggled(GtkToggleToolButton *toggle_tool_button, gpointer user_data)
{
	if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(ToggleSeeParentClonesSoftware))) {
		conf_set(&gnomess_conf,SelectedMachine,"SeeParentClonesSoftware","yes",FALSE);
		enable_parent_clones = 1;
	}
	else {
		conf_set(&gnomess_conf,SelectedMachine,"SeeParentClonesSoftware","no",FALSE);
		enable_parent_clones = 0;
	}
	FillSoftwareList(SelectedMachine,SoftwareTree,mess,&gnomess_conf, enable_parent_clones);
}

/***************************************************************************


  on_RunMachine_clicked 


 ***************************************************************************/
void on_RunMachine_clicked(GtkToolButton *toolbutton,gpointer user_data)
{
	/* Execute mess binary */
	run_mess(TRUE);
}

/***************************************************************************


  on_RunWithDevice_clicked 


 ***************************************************************************/
void on_RunWithDevice_clicked(GtkToolButton *toolbutton,gpointer user_data)
{
	/* Execute mess binary */
	run_mess(FALSE);
}

/***************************************************************************


  on_RandomMachine_clicked 


 ***************************************************************************/
void on_RandomMachine_clicked(GtkToolButton *toolbutton,gpointer user_data)
{
	char path_string[10];
	int path_int = 0;
	GtkTreeModel*   model;
	GtkTreeSelection *sel;
	GtkTreePath *path;

	/* Count the number of systems */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(MachineTree));
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(MachineTree));
	path_int = gtk_tree_model_iter_n_children(model,NULL);

	/* randomly select a machine */
	int random_path = rand() % path_int;
	sprintf(path_string,"%d",random_path);
	path = gtk_tree_path_new_from_string (path_string);
	gtk_tree_selection_select_path (sel, path); 
	gtk_tree_view_expand_to_path( GTK_TREE_VIEW( MachineTree ), path );
	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW( MachineTree ),path,NULL,FALSE,0.0,0.0);
	gtk_tree_view_set_cursor( GTK_TREE_VIEW( MachineTree ), path, NULL, FALSE ); /* launch on_cursor_changed */
	gtk_tree_path_free (path);
}

/***************************************************************************


  on_RandomSoftware_clicked 


 ***************************************************************************/
void on_RandomSoftware_clicked(GtkToolButton *toolbutton,gpointer user_data)
{
	GtkTreeModel*   model;
	int num_software;
	char path_string[10];
	GtkTreePath *path;
	GtkTreeSelection *sel=NULL;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(SoftwareTree));
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(SoftwareTree));

	/* Count the number of softwares */
	num_software = gtk_tree_model_iter_n_children(model,NULL);

	if( num_software == 0 ) return;

	/* randomly select a software */
	int random_path = rand() % num_software;
	sprintf(path_string,"%d",random_path);
	path = gtk_tree_path_new_from_string (path_string);
	gtk_tree_selection_select_path (sel, path);
	gtk_tree_view_expand_to_path( GTK_TREE_VIEW( SoftwareTree ), path );
	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW( SoftwareTree ),path,NULL,FALSE,0.0,0.0);
	gtk_tree_view_set_cursor( GTK_TREE_VIEW( SoftwareTree ), path, NULL, FALSE ); /* launch on_cursor_changed */
	gtk_tree_path_free (path);

	MountSoftware(NULL,NULL);
}

/***************************************************************************


  get_software_name

WARNING, the returned pointer must be freed

Retrieve the selected software, or else the software with "snap" notification
or else any software mounted
 ***************************************************************************/
char * get_software_name()
{
	GtkTreeModel*   model;
	GtkTreeIter iter;
	gchar * software;
	GtkTreeSelection *sel=NULL;
	int snap;

	/* get selected software */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(SoftwareTree));
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(SoftwareTree));
	if(  gtk_tree_selection_count_selected_rows( sel ) ) /* if selection exists*/
	{
		gtk_tree_selection_get_selected(sel,&model,&iter);
		gtk_tree_model_get (model,&iter,0,&software, -1);
		return software;
	}

	/* get "snap" software */
	software= get_snap_software_name();
	if(software != NULL) {
		return software;
	}

	/* search any mounted software */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree));
	if( gtk_tree_model_get_iter_first (model,&iter) == FALSE ) return NULL;
	do {
		gtk_tree_model_get (model,&iter,3,&software,4,&snap, -1);
		if(software!=NULL && strcmp(software,"")!= 0) return software;
		g_free(software);
	}
	while( gtk_tree_model_iter_next(model,&iter) != FALSE );

	return NULL;
}

/***************************************************************************


  on_MachineImage_expose_event 


 ***************************************************************************/
gboolean on_MachineImage_expose_event(GtkWidget *widget,GdkEventExpose *event,gpointer user_data)
{
	GdkPixbuf *pixbuf;
	gint iWidth, iHeight;
	GtkWidget * image=NULL;
	float ratio_x,ratio_y, ratio;

	/* this is a hack to avoid looping on infinite resize event */
	if(MachineAlreadyResized) {
		MachineAlreadyResized=0;
		return FALSE;
	}

	/* get the GdkPixbuf */
/*
	pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(widget));
	if( pixbuf == NULL ) {
		return 1;
	}
*/
	/* Size of the imagea */
/*
	iWidth = gdk_pixbuf_get_width(pixbuf);
	iHeight = gdk_pixbuf_get_height(pixbuf);
	float ratio_x = (float)widget->allocation.width / (float)iWidth;
	float ratio_y = (float)widget->allocation.height / (float)iHeight;

	float ratio = ratio_x < ratio_y ? ratio_x : ratio_y;

	if( ratio > 1.0 || pixbuf == NULL ) {
*/
		image = gtk_image_new_from_file((char *)user_data);
		pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(image));
		iWidth = gdk_pixbuf_get_width(pixbuf);
		iHeight = gdk_pixbuf_get_height(pixbuf);

		ratio_x = (float)widget->allocation.width / (float)iWidth;
		ratio_y = (float)widget->allocation.height / (float)iHeight;

		ratio = ratio_x < ratio_y ? ratio_x : ratio_y;
/*
	}
*/

	/* Create dest GdkPixbuf */
	pixbuf = gdk_pixbuf_scale_simple(pixbuf, iWidth*ratio, iHeight*ratio, GDK_INTERP_HYPER);

	/* Replace old GdkPixbuf by the new one */
	gtk_image_set_from_pixbuf(GTK_IMAGE(widget), pixbuf);

	g_object_unref(pixbuf);	
	gtk_widget_destroy(image);

	MachineAlreadyResized=1;

	return FALSE;
}

/***************************************************************************


  on_SoftwareImage_expose_event 


 ***************************************************************************/
gboolean on_SoftwareImage_expose_event(GtkWidget *widget,GdkEventExpose *event,gpointer user_data)
{
	GdkPixbuf *pixbuf;
	gint iWidth, iHeight;
	GtkWidget * image=NULL;
	float ratio_x,ratio_y,ratio;

	/* this is a hack to avoid looping on infinite resize event */
	if(SoftwareAlreadyResized) {
		SoftwareAlreadyResized=0;
		return FALSE;
	}

	/* get the GdkPixbuf */
/*
	pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(widget));
	if( pixbuf == NULL ) {
		return 1;
	}
*/
	/* Size of the image */
/*
	iWidth = gdk_pixbuf_get_width(pixbuf);
	iHeight = gdk_pixbuf_get_height(pixbuf);

	float ratio_x = (float)widget->allocation.width / (float)iWidth;
	float ratio_y = (float)widget->allocation.height / (float)iHeight;

	float ratio = ratio_x < ratio_y ? ratio_x : ratio_y;

	if( ratio > 1.0 || pixbuf == NULL ) {
*/
		image = gtk_image_new_from_file((char *)user_data);
		pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(image));
		iWidth = gdk_pixbuf_get_width(pixbuf);
		iHeight = gdk_pixbuf_get_height(pixbuf);

		ratio_x = (float)widget->allocation.width / (float)iWidth;
		ratio_y = (float)widget->allocation.height / (float)iHeight;

		ratio = ratio_x < ratio_y ? ratio_x : ratio_y;
/*
	}
*/

	/* Create dest GdkPixbuf */
	pixbuf = gdk_pixbuf_scale_simple(pixbuf, iWidth*ratio, iHeight*ratio, GDK_INTERP_TILES);

	/* Replace old GdkPixbuf by the new one */
	gtk_image_set_from_pixbuf(GTK_IMAGE(widget), pixbuf);

	g_object_unref(pixbuf);	
	gtk_widget_destroy(image);

	SoftwareAlreadyResized=1;

	return FALSE;

}
/***************************************************************************


  on_SoftwareImage_button_release_event 


 ***************************************************************************/
gint on_SoftwareImage_button_release_event(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	char * software;
	software = get_software_name();
	if( software != NULL )
		ShowSoftwareImage(GTK_CONTAINER(SoftwareImageContainer),SelectedMachine,mess,software,&gnomess_conf,enable_parent_clones);
	g_free(software);

	return TRUE;
}

/***************************************************************************


  on_MachineImage_button_release_event 


 ***************************************************************************/
gint on_MachineImage_button_release_event(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	ShowMachineImage(GTK_CONTAINER(MachineImageContainer),SelectedMachine,&gnomess_conf);

	return FALSE;
}

/***************************************************************************


  on_menu_configuration


 ***************************************************************************/
gboolean on_menu_configuration(GtkWidget *menuitem, GdkEventButton *event, gpointer user_data)
{
        GtkListStore * ConfigurationListStore;
        GtkTreeModel*   model;
        GtkTreeIter iter;
        gchar * ConfType;
        GtkTreeSelection *sel;
	llist_t * biosset;
	llist_t * ramoption;

        /* get selected configuration */
        sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(ConfigurationTree));
        if( ! gtk_tree_selection_count_selected_rows( sel ) ) /* if no selection */
                return FALSE;
        gtk_tree_selection_get_selected(sel,&model,&iter);
        gtk_tree_model_get (model,&iter,0,&ConfType, -1);

	ConfigurationListStore=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ConfigurationTree)));

	if(strcmp(ConfType,BIOS_TYPE)==0) {
		biosset = (llist_t *)user_data;
		gtk_list_store_set (ConfigurationListStore, &iter,
				0, BIOS_TYPE,
				1, find_attr(biosset,"name"),
				2, find_attr(biosset,"description"),
				-1);
		conf_set(&gnomess_conf,SelectedMachine,BIOS_TYPE,find_attr(biosset,"name"),0);
	}
	else {
		ramoption = (llist_t *)user_data;
		gtk_list_store_set (ConfigurationListStore, &iter,
				0, RAM_TYPE,
				1, get_data(ramoption),
				-1);
		conf_set(&gnomess_conf,SelectedMachine,RAM_TYPE,get_data(ramoption),0);

	}

	g_free(ConfType);

	gtk_menu_shell_deactivate(GTK_MENU_SHELL(gtk_widget_get_parent(menuitem)));

	return TRUE;
}

/***************************************************************************


  ConfigurationTree_do_popup_menu 


 ***************************************************************************/

static void ConfigurationTree_do_popup_menu (GtkWidget *widget, GdkEventButton *event)
{
	GtkWidget *menu;
	int button, event_time;
	GtkTreePath * path;
        GtkTreeModel*   model;
	GtkTreeSelection *sel;

	if(gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(ConfigurationTree),event->x,event->y,&path,NULL,NULL,NULL)==FALSE) {
		return;
	}

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(ConfigurationTree));
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(ConfigurationTree));
	gtk_tree_selection_select_path (sel, path); 
	gtk_tree_view_expand_to_path( GTK_TREE_VIEW( ConfigurationTree ), path );

	menu = gtk_menu_new ();
	g_signal_connect (menu, "deactivate", 
			G_CALLBACK (gtk_widget_destroy), NULL);

	/* ... add menu items ... */
	AddConfigurationMenuItem(SelectedMachine,ConfigurationTree,menu,mess,&gnomess_conf);

	if (event)
	{
		button = event->button;
		event_time = event->time;
	}
	else
	{
		button = 0;
		event_time = gtk_get_current_event_time ();
	}

	gtk_menu_attach_to_widget (GTK_MENU (menu), widget, NULL);
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
			button, event_time);
}

/***************************************************************************


  on_ConfigurationTree_popup_menu 


 ***************************************************************************/
gboolean on_ConfigurationTree_popup_menu(GtkWidget *widget)
{
  ConfigurationTree_do_popup_menu (widget, NULL);
  return TRUE;
}
/***************************************************************************


  on_ConfigurationTree_button_release_event 


 ***************************************************************************/
gint on_ConfigurationTree_button_release_event(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	/* test bouton droit */
	if (event->button == 3 && event->type == GDK_BUTTON_RELEASE)
	{
		ConfigurationTree_do_popup_menu (widget, event);
		return TRUE;
	}
	else if( event->button == 1 ) {
		UpdateConfigurationList(SelectedMachine,ConfigurationTree,mess,&gnomess_conf);
	}

	return TRUE;
}

/***************************************************************************


  on_menu_software


 ***************************************************************************/
gboolean on_menu_software(GtkWidget *menuitem, GdkEventButton *event, gpointer user_data)
{
        GtkTreeModel*   model;
        GtkTreeIter iter;
        GtkTreeSelection *sel;
	gchar * software_name;
	char * device;

        /* get selected software */
        sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(SoftwareTree));
        if( ! gtk_tree_selection_count_selected_rows( sel ) ) /* if no selection */
                return FALSE;
        gtk_tree_selection_get_selected(sel,&model,&iter);
	gtk_tree_model_get (model,&iter,1,&software_name, -1);

	device = (char *)user_data;
	MountSoftware(device,software_name);

	g_free(software_name);

	gtk_menu_shell_deactivate(GTK_MENU_SHELL(gtk_widget_get_parent(menuitem)));

	return TRUE;
}

/***************************************************************************


  SoftwareTree_do_popup_menu 


 ***************************************************************************/

static void SoftwareTree_do_popup_menu (GtkWidget *widget, GdkEventButton *event)
{
	GtkWidget *menu;
	int button, event_time;
	GtkTreePath * path;
        GtkTreeModel*   model;
	GtkTreeIter iter;
	GtkTreeSelection *sel;
	gchar *	software_extension;
	GtkWidget * menu_item;
	char tmp[1024];

	if(gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(SoftwareTree),event->x,event->y,&path,NULL,NULL,NULL)==FALSE) {
		return;
	}
	/* set the selection */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(SoftwareTree));
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(SoftwareTree));
	gtk_tree_selection_select_path (sel, path); 
	gtk_tree_view_expand_to_path( GTK_TREE_VIEW( SoftwareTree ), path );

        /* get selected software */
        sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(SoftwareTree));
        if( ! gtk_tree_selection_count_selected_rows( sel ) ) /* if no selection */
                return;
        gtk_tree_selection_get_selected(sel,&model,&iter);
	gtk_tree_model_get (model,&iter,2,&software_extension,-1);

	if(software_extension==NULL) return;

	str_tolower(software_extension);

	/* Create menu */
	menu = gtk_menu_new ();
	g_signal_connect (menu, "deactivate", 
			G_CALLBACK (gtk_widget_destroy), NULL);

	/* ... add menu items ... */
	int found=0;
	llist_t * machine;
	llist_t * device;
	llist_t * extension;
	llist_t * instance;

	machine = find_node_by_attr(mess,"machine","name",SelectedMachine);
	if( machine == NULL ) return;

	device = find_first_node(machine,"device");
	while( device != NULL ) {
		instance = find_first_node(device,"instance");
		extension = find_node_by_attr(device,"extension","name",software_extension);
		if( extension && instance ) {
			found=1;
			sprintf(tmp,"mount on %s",find_attr(instance,"name"));
			menu_item=gtk_menu_item_new_with_label(tmp);
			gtk_widget_show(menu_item);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
			g_signal_connect(GTK_OBJECT(menu_item), "button-release-event",
					G_CALLBACK(on_menu_software),find_attr(instance,"name"));
		}
		device=find_next_node(device);
	}

	g_free(software_extension);

	/* No contextual menu if no matching extension */
	if(!found) {
		gtk_widget_destroy(menu);
		return;
	}

	if (event)
	{
		button = event->button;
		event_time = event->time;
	}
	else
	{
		button = 0;
		event_time = gtk_get_current_event_time ();
	}

	gtk_menu_attach_to_widget (GTK_MENU (menu), widget, NULL);
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
			button, event_time);

}

/***************************************************************************


  on_SoftwareTree_popup_menu 


 ***************************************************************************/
gboolean on_SoftwareTree_popup_menu(GtkWidget *widget)
{
  SoftwareTree_do_popup_menu (widget, NULL);
  return TRUE;
}
/***************************************************************************


  on_SoftwareTree_button_press_event 


 ***************************************************************************/
gint on_SoftwareTree_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
 	/* test double click */
	if ( event->type==GDK_2BUTTON_PRESS || event->type==GDK_3BUTTON_PRESS) {
		MountSoftware(NULL,NULL);
		/* Execute mess binary */
		run_mess(FALSE);
		return TRUE;
	}
	/* test right click */
	else if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
	{
		SoftwareTree_do_popup_menu (widget, event);
		return TRUE;
	}
	return FALSE;
}

/***************************************************************************


  on_MachineTree_button_press_event 


 ***************************************************************************/
gint on_MachineTree_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
 	/* test double click */
	if ( event->type!=GDK_2BUTTON_PRESS && event->type!=GDK_3BUTTON_PRESS) 
		return FALSE;

	on_RunMachine_clicked(NULL,NULL);

	return FALSE;
}

/***************************************************************************


  MountOnDeviceDialogBox 


 ***************************************************************************/
void MountOnDeviceDialogBox()
{
	GtkTreeModel*	model;
	GtkTreeIter	iter;
	char * device_name;

	GtkTreeSelection *sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(DeviceTree));
	/* if no selection */
	if( ! gtk_tree_selection_count_selected_rows( sel ) )
		return;

	gtk_tree_selection_get_selected(sel,&model,&iter);

	GtkWidget *dialog;
	dialog = gtk_file_chooser_dialog_new ("Open File",
			GTK_WINDOW(MainWindow),
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
	{
		char *filename;
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		gtk_list_store_set (GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree))), &iter,2, filename,3,SoftwareShortName(filename),-1);
		gtk_tree_model_get (model,&iter,1,&device_name, -1);
		conf_set(&gnomess_conf,SelectedMachine,device_name,filename,FALSE);
		g_free (device_name);
		g_free (filename);
	}
	gtk_widget_destroy (dialog);
}
/***************************************************************************


  menu_mount


 ***************************************************************************/
gboolean on_menu_mount(GtkMenuItem *menuitem,gpointer user_data)
{
	MountOnDeviceDialogBox();
	return TRUE;
}
/***************************************************************************


  menu_unmount 


 ***************************************************************************/
gboolean on_menu_unmount(GtkMenuItem *menuitem,gpointer user_data)
{
	GtkTreeModel*	model;
	GtkTreeIter	iter;
	char * device_name;

	GtkTreeSelection *sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(DeviceTree));
	/* if no selection */
	if( ! gtk_tree_selection_count_selected_rows( sel ) )
		return FALSE;

	gtk_tree_selection_get_selected(sel,&model,&iter);

	gtk_list_store_set (GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree))), &iter,2, NULL,3,NULL,-1);

	gtk_tree_model_get (model,&iter,1,&device_name, -1);
	conf_set(&gnomess_conf,SelectedMachine,device_name,NULL,FALSE);
	g_free (device_name);

	gtk_menu_shell_deactivate(GTK_MENU_SHELL(gtk_widget_get_parent(GTK_WIDGET(menuitem))));

	return TRUE;
}
/***************************************************************************


  DeviceTree_do_popup_menu 


 ***************************************************************************/

static void DeviceTree_do_popup_menu (GtkWidget *widget, GdkEventButton *event)
{
	GtkWidget *menu;
	int button, event_time;
	GtkTreePath * path;
        GtkTreeModel*   model;
	GtkTreeSelection *sel;
	GtkWidget * menu_mount;
	GtkWidget * menu_unmount;

	if(gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(DeviceTree),event->x,event->y,&path,NULL,NULL,NULL)==FALSE) {
		return;
	}

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree));
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(DeviceTree));
	gtk_tree_selection_select_path (sel, path); 
	gtk_tree_view_expand_to_path( GTK_TREE_VIEW( DeviceTree ), path );

	menu = gtk_menu_new ();
	g_signal_connect (menu, "deactivate", 
			G_CALLBACK (gtk_widget_destroy), NULL);

	/* ... add menu items ... */
	menu_mount=gtk_menu_item_new_with_label ("mount");
	gtk_widget_show(menu_mount);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_mount);
	g_signal_connect(GTK_OBJECT(menu_mount), "button-release-event",
			G_CALLBACK(on_menu_mount), NULL);

	menu_unmount=gtk_menu_item_new_with_label ("unmount");
	gtk_widget_show(menu_unmount);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_unmount);
	g_signal_connect(GTK_OBJECT(menu_unmount), "button-release-event",
			G_CALLBACK(on_menu_unmount), NULL);

	if (event)
	{
		button = event->button;
		event_time = event->time;
	}
	else
	{
		button = 0;
		event_time = gtk_get_current_event_time ();
	}

	gtk_menu_attach_to_widget (GTK_MENU (menu), widget, NULL);
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
			button, event_time);
}

/***************************************************************************


  on_DeviceTree_popup_menu 


 ***************************************************************************/
gboolean on_DeviceTree_popup_menu(GtkWidget *widget)
{
  DeviceTree_do_popup_menu (widget, NULL);
  return TRUE;
}

/***************************************************************************


  on_DeviceSnapToggled 


 ***************************************************************************/
gint on_DeviceSnapToggled(GtkCellRendererToggle *cell_renderer, gchar *path, gpointer user_data)
{
	GtkTreeModel*   model;
	GtkTreeIter     iter;
	gchar * device;

	/* FIXME: this is crapy "radio-button" behavior */
	UnselectAllSnap(DeviceTree);

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree));
	gtk_tree_model_get_iter_from_string(model, &iter, path);
	gtk_list_store_set(GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(DeviceTree))),&iter,4,1,-1);
	gtk_tree_model_get(model,&iter,1,&device,-1);
	conf_set(&gnomess_conf,SelectedMachine,"snap",device,FALSE);
	g_free(device);

	return TRUE;
}

/***************************************************************************


  on_DeviceTree_button_press_event 


 ***************************************************************************/
gint on_DeviceTree_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	/* test right click */
	if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
	{
		DeviceTree_do_popup_menu (widget, event);
		return TRUE;
	}

	return FALSE;
}
/***************************************************************************


  on_SoftwareTree_cursor_changed 


 ***************************************************************************/

void on_SoftwareTree_cursor_changed(GtkTreeView *tree_view,gpointer user_data)
{
	gchar *	software_name;
	GtkTreeModel*	model;
	GtkTreeIter	iter;

	GtkTreeSelection *sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(SoftwareTree));
	/* if no selection */
	if( ! gtk_tree_selection_count_selected_rows( sel ) )
		return;
	gtk_tree_selection_get_selected(sel,&model,&iter);
	gtk_tree_model_get (model,&iter,0,&software_name, -1);

	ShowSoftwareImage(GTK_CONTAINER(SoftwareImageContainer),SelectedMachine,mess,software_name,&gnomess_conf,enable_parent_clones);

	g_free(software_name);
}

/***************************************************************************


  on_MachineTree_cursor_changed 


 ***************************************************************************/

void on_MachineTree_cursor_changed(GtkTreeView *tree_view,gpointer user_data)
{
	gchar *	machine_name;
	GtkTreeModel*	model;
	GtkTreeIter	iter;
	char tmp[1024];

	GtkTreeSelection *sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(MachineTree));
	/* if no selection */
	if( ! gtk_tree_selection_count_selected_rows( sel ) )
		return;

	gtk_tree_selection_get_selected(sel,&model,&iter);

	gtk_tree_model_get (model,&iter,0,&machine_name, -1);

	if(SelectedMachine != NULL) free(SelectedMachine);
	SelectedMachine=strdup(machine_name);
	g_free(machine_name);

	/* set the mouse cursor */
        GdkCursor* cursor;
        cursor = gdk_cursor_new(GDK_CLOCK);
	if( GDK_IS_WINDOW (MainWindow->window) ) {
	        gdk_window_set_cursor(MainWindow->window, cursor);
	}

	FillDeviceList(SelectedMachine,DeviceTree,mess);
	FillSoftwareList(SelectedMachine,SoftwareTree,mess,&gnomess_conf, enable_parent_clones);

	FillConfigurationList(SelectedMachine,ConfigurationTree,mess,&gnomess_conf);
	MountSoftwareInConfFile(SelectedMachine,&gnomess_conf,mess);
	SetSnapSelection(SelectedMachine,DeviceTree,&gnomess_conf);
	SetToggleSeeParentClonesSoftware();
	ShowMachineImage(GTK_CONTAINER(MachineImageContainer),SelectedMachine,&gnomess_conf);
        char * software;
        software = get_software_name();
        if( software != NULL ) {
                ShowSoftwareImage(GTK_CONTAINER(SoftwareImageContainer),SelectedMachine,mess,software,&gnomess_conf,enable_parent_clones);
	}
	else {
		GtkWidget * previousImage = gtk_bin_get_child(GTK_BIN(SoftwareImageContainer));
		if( previousImage) {
			gtk_widget_destroy(previousImage);
		}
	}
        g_free(software);
	conf_set(&gnomess_conf,"gnomess","selected_machine",SelectedMachine,FALSE);

        /* Update status bar */
	if(conf_done) {
		llist_t * machine;
		llist_t * description;
		llist_t * driver;

		machine = find_node_by_attr(mess,"machine","name",SelectedMachine);
		if(machine) {
			driver = find_first_node(machine,"driver");
			description = find_first_node(machine,"description");

			if( driver && description ) {
				sprintf(tmp,"%s , Status : %s , Emulation : %s , Color : %s , Sound : %s , Graphic : %s , Savestate : %s", get_data(description), find_attr(driver,"status"), find_attr(driver,"emulation"), find_attr(driver,"color"), find_attr(driver,"sound"), find_attr(driver,"graphic"), find_attr(driver,"savestate"));
				guint id;
				id=gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar),"general");
				gtk_statusbar_pop(GTK_STATUSBAR(StatusBar),id);
				gtk_statusbar_push(GTK_STATUSBAR(StatusBar),id,tmp);
			}
		}
	}

	/* set back the mouse cursor */
        gdk_cursor_destroy(cursor);
	if( GDK_IS_WINDOW (MainWindow->window) ) {
        	gdk_window_set_cursor(MainWindow->window, NULL);
	}
}

/***************************************************************************


  on_About_clicked 


 ***************************************************************************/
void on_About_clicked(GtkToolButton *toolbutton,gpointer user_data)
{
	GtkWidget * dialog;
	dialog = gtk_message_dialog_new (GTK_WINDOW(MainWindow),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_CLOSE,
			"GNOMESS version %s", VERSION);
	gtk_message_dialog_format_secondary_text ( GTK_MESSAGE_DIALOG(dialog), "http://bobz38.free.fr/gnomess/" );
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
}

/***************************************************************************


  on_AddPath_clicked 


 ***************************************************************************/
void on_AddPath_clicked(GtkToolButton *toolbutton,gpointer user_data)
{
	GtkTreeModel*   model;
	GtkTreeIter iter;
	char tmp[1024];

        char *in=NULL;
        char *out=NULL;

	/* Fill data */
	/* MESS path */	
	const char *dummy = conf_get(&gnomess_conf,"mess","path");
	if( dummy ) {
		gtk_entry_set_text(GTK_ENTRY(PathMessEntry),dummy);
	}

	/* Software path */

	model=gtk_tree_view_get_model(GTK_TREE_VIEW(PathSoftwareList));
	gtk_list_store_clear(GTK_LIST_STORE(model));

	dummy = conf_get(&gnomess_conf,"mess","software_path");
	if( dummy == NULL || strcmp(dummy,"")==0 ) {
		/* if no software path, use the MESS binary path */
		out = strdup(conf_get(&gnomess_conf,"mess","path"));
		/* keep only the path to the binary */
		int i;
		for(i= strlen(out); i >= 0 ; i--) {
			if(out[i] == '/') {
				out[i] = 0;
				break;
			}
		}
		sprintf(tmp,"%s/%s",out,"software");
		gtk_list_store_append (GTK_LIST_STORE(model), &iter);
		gtk_list_store_set (GTK_LIST_STORE(model), &iter,0,tmp,-1);
		free(out);
	}
	else {
		/* Get each software path in software_path*/
		out = NULL;
		in=strdup(dummy);
		if(in == NULL) return;
		out = strtok(in,";");

		while(out!=NULL) {
			gtk_list_store_append (GTK_LIST_STORE(model), &iter);
			gtk_list_store_set (GTK_LIST_STORE(model), &iter,0,out,-1);

			out = strtok(NULL,";");
		}
	}

	/* Machine snapshot path */

	model=gtk_tree_view_get_model(GTK_TREE_VIEW(PathMachineSnapList));
	gtk_list_store_clear(GTK_LIST_STORE(model));

	dummy = conf_get(&gnomess_conf,"mess","machine_snap_path");
	if( dummy == NULL || strcmp(dummy,"")==0 ) {
		/* if no machine snap path, use the MESS binary path */
		out = strdup(conf_get(&gnomess_conf,"mess","path"));
		/* keep only the path to the binary */
		int i;
		for(i= strlen(out); i >= 0 ; i--) {
			if(out[i] == '/') {
				out[i] = 0;
				break;
			}
		}
		#define NUM_DIR 6
		char sub_directory[NUM_DIR][9] = {"cabinets","cpanel","flyers","marquees","pcb","titles"};
		for(i=0; i<NUM_DIR ; i++) {
			sprintf(tmp,"%s/%s",out,sub_directory[i]);
			gtk_list_store_append (GTK_LIST_STORE(model), &iter);
			gtk_list_store_set (GTK_LIST_STORE(model), &iter,0,tmp,-1);
		}
		free(out);
	}
	else {
		/* Get each machine snap path in machine_snap_path */
		out = NULL;
		in=strdup(dummy);
		if(in == NULL) return;
		out = strtok(in,";");

		while(out!=NULL) {
			gtk_list_store_append (GTK_LIST_STORE(model), &iter);
			gtk_list_store_set (GTK_LIST_STORE(model), &iter,0,out,-1);

			out = strtok(NULL,";");
		}
	}

	/* Software snapshot path */

	model=gtk_tree_view_get_model(GTK_TREE_VIEW(PathSoftwareSnapList));
	gtk_list_store_clear(GTK_LIST_STORE(model));

	dummy = conf_get(&gnomess_conf,"mess","software_snap_path");
	if( dummy == NULL || strcmp(dummy,"")==0 ) {
		/* if no software snap path, use the MESS binary path */
		out = strdup(conf_get(&gnomess_conf,"mess","path"));
		/* keep only the path to the binary */
		int i;
		for(i= strlen(out); i >= 0 ; i--) {
			if(out[i] == '/') {
				out[i] = 0;
				break;
			}
		}

		sprintf(tmp,"%s/%s",out,"snap");
		gtk_list_store_append (GTK_LIST_STORE(model), &iter);
		gtk_list_store_set (GTK_LIST_STORE(model), &iter,0,tmp,-1);
		free(out);
	}
	else {
		/* Get each software snap path in software_snap_path */
		out = NULL;
		in=strdup(dummy);
		if(in == NULL) return;
		out = strtok(in,";");

		while(out!=NULL) {
			gtk_list_store_append (GTK_LIST_STORE(model), &iter);
			gtk_list_store_set (GTK_LIST_STORE(model), &iter,0,out,-1);

			out = strtok(NULL,";");
		}
	}

	gtk_widget_show_all(PathWindow);

	/* Update the lists */
}

/***************************************************************************


  on_PathCancel_released 


 ***************************************************************************/
gint on_PathCancel_released(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	gtk_widget_hide_all(PathWindow);
	
	return TRUE;
}

/***************************************************************************


  on_PathMessButton_released 


 ***************************************************************************/
gint on_PathMessButton_released(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	GtkWidget *dialog;
	dialog = gtk_file_chooser_dialog_new ("Select MESS executable path",
			GTK_WINDOW(PathWindow),
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);
	gint result = gtk_dialog_run (GTK_DIALOG (dialog));
	if (result == GTK_RESPONSE_ACCEPT)
	{
		char * foo;
		foo = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		gtk_entry_set_text(GTK_ENTRY(PathMessEntry),foo);
		g_free (foo);
	}

	gtk_widget_destroy (dialog);

	return TRUE;
}


/***************************************************************************


  add_path_list 


 ***************************************************************************/

void add_path_list(GtkTreeView * view) {

	GtkWidget *dialog;
	GtkTreeModel*   model;
	GtkTreeIter iter;

	model=gtk_tree_view_get_model(view);

	dialog = gtk_file_chooser_dialog_new ("Select path",
			GTK_WINDOW(PathWindow),
			GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);
	gint result = gtk_dialog_run (GTK_DIALOG (dialog));
	if (result == GTK_RESPONSE_ACCEPT)
	{
		char * foo;
		foo = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		gtk_list_store_append (GTK_LIST_STORE(model), &iter);
		gtk_list_store_set (GTK_LIST_STORE(model), &iter,0,foo,-1);
		g_free (foo);
	}

	gtk_widget_destroy (dialog);

}

/***************************************************************************


  on_PathSoftwareAdd_released 


 ***************************************************************************/
gint on_PathSoftwareAdd_released(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	add_path_list( GTK_TREE_VIEW(PathSoftwareList) );
	return TRUE;
}

/***************************************************************************


  on_PathMachineSnapOpen_released 


 ***************************************************************************/
gint on_PathMachineSnapOpen_released(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	add_path_list( GTK_TREE_VIEW(PathMachineSnapList) );
	return TRUE;
}

/***************************************************************************


  on_PathSoftwareSnapAdd_released 


 ***************************************************************************/
gint on_PathSoftwareSnapAdd_released(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	add_path_list( GTK_TREE_VIEW(PathSoftwareSnapList) );
	return TRUE;
}

/***************************************************************************


  remove_path_list 


 ***************************************************************************/
void remove_path_list(GtkTreeView * view) {
        GtkTreeModel*   model;
        GtkTreeIter iter;
        GtkTreeSelection *sel;

        model = gtk_tree_view_get_model(view);
        sel=gtk_tree_view_get_selection(view);
        if( gtk_tree_model_get_iter_first (model,&iter) == FALSE ) return;
        do {

                if( gtk_tree_selection_iter_is_selected (sel,&iter)) {
                        gtk_list_store_remove(GTK_LIST_STORE(model),&iter);
                        break;
                }
        }
        while( gtk_tree_model_iter_next(model,&iter) != FALSE );
}

/***************************************************************************


  on_PathSoftwareRemove_released 


 ***************************************************************************/
gint on_PathSoftwareRemove_released(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	remove_path_list(GTK_TREE_VIEW(PathSoftwareList));
	return TRUE;
}

/***************************************************************************


  on_PathMachineSnapRemove_released 


 ***************************************************************************/
gint on_PathMachineSnapRemove_released(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	remove_path_list(GTK_TREE_VIEW(PathMachineSnapList));
	return TRUE;
}
/***************************************************************************


  on_PathSoftwareSnapRemove_released 


 ***************************************************************************/
gint on_PathSoftwareSnapRemove_released(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
	remove_path_list(GTK_TREE_VIEW(PathSoftwareSnapList));
	return TRUE;
}

/***************************************************************************


  on_PathOK_released 


 ***************************************************************************/
gint on_PathOK_released(GtkWidget *widget, GdkEventButton *event, gpointer func_data)
{
        GtkTreeModel*   model;
        GtkTreeIter iter;
	gchar * tmp;

	/* MESS binary */
	tmp = (gchar *)gtk_entry_get_text(GTK_ENTRY(PathMessEntry));
	if(tmp != NULL && strcmp(tmp,"")) {
		conf_set(&gnomess_conf,"mess","path",tmp,FALSE);
	}
	else {
		GtkWidget * dialog;
		dialog = gtk_message_dialog_new (GTK_WINDOW(PathWindow),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_ERROR,
				GTK_BUTTONS_CLOSE,
				"MESS path executable can't be empty");
//		gtk_message_dialog_format_secondary_text ( GTK_MESSAGE_DIALOG(dialog), "..." );
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		return TRUE;
	}

	/* Software paths */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(PathSoftwareList));
	if( gtk_tree_model_get_iter_first (model,&iter) == FALSE ) {
		conf_set(&gnomess_conf,"mess","software_path","",FALSE);
	}
	else {
		gtk_tree_model_get (model,&iter,0,&tmp,-1);
		conf_set(&gnomess_conf,"mess","software_path",tmp,FALSE);
		g_free(tmp);
		while( gtk_tree_model_iter_next(model,&iter) != FALSE ) {
			gtk_tree_model_get (model,&iter,0,&tmp,-1);
			conf_set(&gnomess_conf,"mess","software_path",tmp,TRUE);
			g_free(tmp);
		}
	}

	/* Machine snap paths */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(PathMachineSnapList));
	if( gtk_tree_model_get_iter_first (model,&iter) == FALSE ) {
		conf_set(&gnomess_conf,"mess","machine_snap_path","",FALSE);
	}
	else {
		gtk_tree_model_get (model,&iter,0,&tmp,-1);
		conf_set(&gnomess_conf,"mess","machine_snap_path",tmp,FALSE);
		g_free(tmp);
		while( gtk_tree_model_iter_next(model,&iter) != FALSE ) {
			gtk_tree_model_get (model,&iter,0,&tmp,-1);
			conf_set(&gnomess_conf,"mess","machine_snap_path",tmp,TRUE);
			g_free(tmp);
		}
	}

	/* Software snap paths */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(PathSoftwareSnapList));
	if( gtk_tree_model_get_iter_first (model,&iter) == FALSE ) {
		conf_set(&gnomess_conf,"mess","software_snap_path","",FALSE);
	}
	else {
		gtk_tree_model_get (model,&iter,0,&tmp,-1);
		conf_set(&gnomess_conf,"mess","software_snap_path",tmp,FALSE);
		g_free(tmp);
		while( gtk_tree_model_iter_next(model,&iter) != FALSE ) {
			gtk_tree_model_get (model,&iter,0,&tmp,-1);
			conf_set(&gnomess_conf,"mess","software_snap_path",tmp,TRUE);
			g_free(tmp);
		}
	}

	gtk_widget_hide_all(PathWindow);

	/* Update all lists */
	on_MachineTree_cursor_changed(NULL,NULL);

	return TRUE;
}

/***************************************************************************


  on_MainWindow_destroy 


 ***************************************************************************/
gboolean on_MainWindow_configure_event(GtkWidget *widget,GdkEventConfigure *event,gpointer user_data)
{
	char tmp[128];

	sprintf(tmp,"%d",event->width);
	conf_set(&gnomess_conf,"gnomess","main_width",tmp,FALSE);
	sprintf(tmp,"%d",event->height);
	conf_set(&gnomess_conf,"gnomess","main_height",tmp,FALSE);

	return FALSE;
}

void on_MainWindow_destroy(GtkObject *object, gpointer user_data)
{
	char tmp[128];

	sprintf(tmp,"%d",gtk_paned_get_position(GTK_PANED(hpaned_machine)));
	conf_set(&gnomess_conf,"gnomess","machine_hpane",tmp,FALSE);
	sprintf(tmp,"%d",gtk_paned_get_position(GTK_PANED(vpaned_machine)));
	conf_set(&gnomess_conf,"gnomess","machine_vpane",tmp,FALSE);
	sprintf(tmp,"%d",gtk_paned_get_position(GTK_PANED(hpaned_software)));
	conf_set(&gnomess_conf,"gnomess","software_hpane",tmp,FALSE);
	sprintf(tmp,"%d",gtk_paned_get_position(GTK_PANED(vpaned_software_image)));
	conf_set(&gnomess_conf,"gnomess","software_image_vpane",tmp,FALSE);
	sprintf(tmp,"%d",gtk_paned_get_position(GTK_PANED(vpaned_device)));
	conf_set(&gnomess_conf,"gnomess","device_vpane",tmp,FALSE);

	gtk_main_quit();
}
/***************************************************************************


  main 


 ***************************************************************************/
int main (int argc, char **argv)
{
	char tmp[1024];
	int i = 0;
	int first_run = 0;

	srand ( time(NULL) );
	bzero(additional_parameters,1024);

	/* Load conf file */
	conf_file_init(&gnomess_conf);

	/* Start GTK stuff */

	/* load main window */
	GtkBuilder              *builder = NULL;
	//GError			*error = NULL;
	gtk_init (&argc, &argv);
	builder = gtk_builder_new ();

	sprintf(tmp,"%s/gnomess/gnomess.glade",DATADIR);
	if( ! gtk_builder_add_from_file (builder, tmp,NULL)) {
		g_print("Error while loading UI file gnomess.glade\n");
		return 1;
	}

	MainWindow = GTK_WIDGET(gtk_builder_get_object(builder,"MainWindow"));
	MachineTree = GTK_WIDGET(gtk_builder_get_object(builder,"MachineTree"));
	DeviceTree = GTK_WIDGET(gtk_builder_get_object(builder,"DeviceTree"));
	SoftwareTree = GTK_WIDGET(gtk_builder_get_object(builder,"SoftwareTree"));
	ConfigurationTree = GTK_WIDGET(gtk_builder_get_object(builder,"ConfigurationTree"));
	StatusBar = GTK_WIDGET(gtk_builder_get_object(builder,"StatusBar"));
	ToggleFullScreen = GTK_WIDGET(gtk_builder_get_object(builder,"ToggleFullScreen"));
	ToggleMaximize = GTK_WIDGET(gtk_builder_get_object(builder,"ToggleMaximize"));
	ToggleSound = GTK_WIDGET(gtk_builder_get_object(builder,"ToggleSound"));
	ToggleRenameSnap = GTK_WIDGET(gtk_builder_get_object(builder,"ToggleRenameSnap"));
	ToggleSeeParentClonesSoftware = GTK_WIDGET(gtk_builder_get_object(builder,"ToggleSeeParentClonesSoftware"));
	MachineImageContainer = GTK_WIDGET(gtk_builder_get_object(builder,"MachineImageContainer"));
	SoftwareImageContainer = GTK_WIDGET(gtk_builder_get_object(builder,"SoftwareImageContainer"));
	hpaned_machine = GTK_WIDGET(gtk_builder_get_object(builder,"hpaned_machine"));
	vpaned_machine = GTK_WIDGET(gtk_builder_get_object(builder,"vpaned_machine"));
	hpaned_software = GTK_WIDGET(gtk_builder_get_object(builder,"hpaned_software"));
	vpaned_software_image = GTK_WIDGET(gtk_builder_get_object(builder,"vpaned_software_image"));
	vpaned_device = GTK_WIDGET(gtk_builder_get_object(builder,"vpaned_device"));


	gtk_builder_connect_signals (builder, NULL);
	g_object_unref (G_OBJECT (builder));

	/* load path window */
	builder = gtk_builder_new ();

	sprintf(tmp,"%s/gnomess/directory_chooser.glade",DATADIR);
	if( ! gtk_builder_add_from_file (builder, tmp,NULL)) {
		g_print("Error while loading UI file directory_chooser.glade\n");
		return 1;
	}

	PathWindow = GTK_WIDGET(gtk_builder_get_object(builder,"PathWindow"));
	PathCancel = GTK_WIDGET(gtk_builder_get_object(builder,"PathCancel"));
	PathMessEntry = GTK_WIDGET(gtk_builder_get_object(builder,"PathMessEntry"));
	PathSoftwareList = GTK_WIDGET(gtk_builder_get_object(builder,"PathSoftwareList"));
	PathMachineSnapList = GTK_WIDGET(gtk_builder_get_object(builder,"PathMachineSnapList"));
	PathSoftwareSnapList = GTK_WIDGET(gtk_builder_get_object(builder,"PathSoftwareSnapList"));

	gtk_builder_connect_signals (builder, NULL);
	g_object_unref (G_OBJECT (builder));


	/* Get MESS data*/
	const char *dummy = conf_get(&gnomess_conf,"mess","path");
	mess = LoadMessData(dummy);
	/* ask for the MESS binary if needed */
	while( mess == NULL) {
		GtkWidget *dialog;

		first_run = 1;

		dialog = gtk_file_chooser_dialog_new ("Select MESS binary",
				GTK_WINDOW(MainWindow),
				GTK_FILE_CHOOSER_ACTION_OPEN,
				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
				NULL);
		gint result = gtk_dialog_run (GTK_DIALOG (dialog));
		if (result == GTK_RESPONSE_ACCEPT)
		{
			char * foo;
			foo = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
			mess = LoadMessData(foo);
			conf_set(&gnomess_conf,"mess","path",foo,FALSE);

			g_free (foo);
		}
		else if (result == GTK_RESPONSE_CANCEL)
			exit(1);

		gtk_widget_destroy (dialog);
	}

	/* set the working directory to MESS one so that no MESS file are created in the current directory */
	strcpy(tmp,conf_get(&gnomess_conf,"mess","path"));
	i=strlen(tmp)-1;
	while(tmp[i] != '/') i--;
	tmp[i] = 0;
	int ret = chdir(tmp);
	ret = ret; /*avoid a compile time warning */
	/* Save this directory as a default base directory */
	conf_set(&gnomess_conf,"mess","base_path",tmp,TRUE);


	/* Fill machine list*/
	int machine_num,machine_clone_num;
	FillMachineList(MachineTree,mess,&machine_num, &machine_clone_num);

        /* Update status bar */
        sprintf(tmp,"MESS %s - Machines : %d - Clones : %d - Total : %d", find_attr(mess,"build"), machine_num, machine_clone_num, machine_num+machine_clone_num);
	guint id;
        id=gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar),"general");
        gtk_statusbar_pop(GTK_STATUSBAR(StatusBar),id);
        gtk_statusbar_push(GTK_STATUSBAR(StatusBar),id,tmp);


	conf_start(&gnomess_conf);
	update_additional_parameters(&gnomess_conf);

	gtk_window_set_icon_name(GTK_WINDOW(MainWindow),"gnomess");

	gtk_widget_show_all(MainWindow);

	if( first_run == 1 ) {
		on_AddPath_clicked(NULL,NULL);
	}

	conf_done = 1;

	gtk_main ();

	/* Write conf file */
	conf_exit(&gnomess_conf);

	return 0;
}
