#include <gtk/gtk.h>
#include "parse_data.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "conf_file.h"

extern gint on_MachineImage_button_release_event(GtkWidget *widget, GdkEventButton *event, gpointer func_data);
gboolean on_MachineImage_expose_event(GtkWidget *widget,GdkEventExpose *event,gpointer user_data);
static int current_image_path = 0;


void FillMachineList(GtkWidget * MachineTree, llist_t * mess,int * machine_num, int * machine_clone_num)
{
        GtkTreeIter iter;
	GtkTreeIter parent_iter;
	GtkTreeIter previous_root_iter;
	int parent_found = 0;
	GtkTreeStore * MachineTreeStore;
	int machine_number = 0;
	int machine_clone_number = 0;
	llist_t * machine;

        MachineTreeStore=GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(MachineTree)));
	machine = find_first_node(mess,"machine");
	while ( machine != NULL ) {
		/* Search parent if any */
		parent_found = 0;
		if( find_attr(machine,"cloneof") != NULL ) {
			GtkTreeModel*   model;
			gchar * parent_name;

			model = gtk_tree_view_get_model(GTK_TREE_VIEW(MachineTree));

			/* First try the previous root iter */
			gtk_tree_model_get (model,&previous_root_iter,0,&parent_name,-1);
			if(strcmp( parent_name, find_attr(machine,"cloneof")) == 0 ) {
				parent_found = 1;
				parent_iter = previous_root_iter;
				g_free(parent_name);
			}

			if( ! parent_found ) {
				//			sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(MachineTree));
				if( gtk_tree_model_get_iter_first (model,&parent_iter) != FALSE ) {
					do {
						gtk_tree_model_get (model,&parent_iter,0,&parent_name,-1);
						if(strcmp( parent_name, find_attr(machine,"cloneof")) == 0 ) {
							parent_found = 1;
							g_free(parent_name);
							break;
						}
						g_free(parent_name);
					}
					while( gtk_tree_model_iter_next(model,&parent_iter) != FALSE );
				}
			}
		}

		/* Add the machine */
		if( parent_found ) {
			gtk_tree_store_append (MachineTreeStore, &iter, &parent_iter);
			machine_clone_number++;
		}
		else {
			gtk_tree_store_append (MachineTreeStore, &iter, NULL);
			previous_root_iter=iter; /* to speed up the parent search */
			machine_number++;
		}
                gtk_tree_store_set (MachineTreeStore, &iter,
                        0, find_attr(machine,"name"),
                        1, get_data(find_first_node(machine,"description")),
                        3, get_data(find_first_node(machine,"year")),
                        4, get_data(find_first_node(machine,"manufacturer")),
                        -1);

		machine=find_next_node(machine);
        }

	if(machine_num != NULL ) {
		*machine_num = machine_number;
	}
	if(machine_clone_num != NULL ) {
		*machine_clone_num = machine_clone_number;
	}
}

static void add_image_path(char ***paths,int * paths_num,char * base,const char * machine){
	#define NUM_DIR 6
	#define NUM_EXT 12
	char image_path[1024];
	int fd;
	char image_extension[NUM_EXT][9] = {".PNG",".png",".Png",".jpg",".Jpg",".JPG",".jpeg",".Jpeg",".JPEG",".gif",".Gif",".GIF"};
	int j;

	for(j=0;j<NUM_EXT;j++) {
		sprintf(image_path,"%s/%s%s",base,machine,image_extension[j]);
		fd=open(image_path,O_RDONLY);
		if(fd != -1) {
			close(fd);
			(*paths_num)++;
			*paths = realloc(*paths,(*paths_num)*sizeof(char *));
			(*paths)[(*paths_num)-1] = malloc(strlen(image_path) + 1);
			strcpy((*paths)[(*paths_num)-1],image_path);
		}
	}

}

void ShowMachineImage(GtkContainer * container, const char * machine, conf_t *conf)
{
	char *in=NULL;
	char *out=NULL;
	char **paths=NULL;
	int paths_number=0;
	const char * dummy;
	GtkWidget *image;
	char * selected_machine;

	/* Get each base path */
	dummy = conf_get(conf,"mess","machine_snap_path");
	if( dummy == NULL ) return;
	out = NULL;
	in=strdup(dummy);
	if(in == NULL) return;
	out = strtok(in,";");	
	while(out!=NULL) {
		add_image_path(&paths,&paths_number,out,machine);
		out = strtok(NULL,";");	
	}

	GList * list = gtk_container_get_children(container);
	if(list != NULL ) {
		if( list->data != NULL )
			gtk_widget_destroy(list->data);
		g_list_free(list);
	}

	if(paths_number > 0) {
		if(paths_number <= current_image_path) {
			current_image_path = 0;
		}

		
		selected_machine = paths[current_image_path];
		current_image_path = (current_image_path + 1) % paths_number;
		image = gtk_image_new_from_file(selected_machine);

                /*Set the image's size to the window's size */
                GtkAllocation allocation;
                gtk_widget_get_allocation (GTK_WIDGET(container),&allocation);


                GdkPixbuf *pSrcPixbuf, *pDestPixbuf;
                gint iWidth, iHeight;

                /* get the GdkPixbuf */
                pSrcPixbuf = gtk_image_get_pixbuf(GTK_IMAGE(image));
                /* Size of the imagea */
                iWidth = gdk_pixbuf_get_width(pSrcPixbuf);
                iHeight = gdk_pixbuf_get_height(pSrcPixbuf);
                float ratio_x = (float)allocation.width / (float)iWidth;
                float ratio_y = (float)allocation.height / (float)iHeight;
                /* Create dest GdkPixbuf */
                if( ratio_x < ratio_y )
                        pDestPixbuf = gdk_pixbuf_scale_simple(pSrcPixbuf, iWidth*ratio_x, iHeight*ratio_x, GDK_INTERP_HYPER);
                else
                        pDestPixbuf = gdk_pixbuf_scale_simple(pSrcPixbuf, iWidth*ratio_y, iHeight*ratio_y, GDK_INTERP_HYPER);

                /* Replace old GdkPixbuf by the new one */
                gtk_image_set_from_pixbuf(GTK_IMAGE(image), pDestPixbuf);

		g_object_unref(pDestPixbuf);

		g_signal_connect(image, "expose-event", G_CALLBACK(on_MachineImage_expose_event), (gpointer)selected_machine);

                /* Add to eventbox to catch event on the image */
                gtk_container_add (GTK_CONTAINER (container), image);

		gtk_widget_show_all (GTK_WIDGET(container));
	}
}
