#include <expat.h>
#include <string.h>
#include "linked_list.h"
#include <stdio.h>


XML_Parser parser;
llist_t * root_node;



/****************************************************************************

        Start Element

****************************************************************************/
void ElementStart(void *userData,const XML_Char *name,const XML_Char **atts)
{
	int i = 0;
	llist_t * list;

        /* Get the target node */
        if( userData == NULL ) {
                list = new_llist();
                root_node = list; /*root node*/
        }
        else {
                list = (llist_t *)userData;
                list = add_child(list, new_llist());
        }

	/* add node name */
	add_name(list,strdup(name));

	/* add attributes name */
	llist_t * attr;
	while( atts[i] != NULL ) {
		attr = new_llist();
		add_name(attr,strdup(atts[i++]));
		add_data(attr,strdup(atts[i++]));
		add_attr(list,attr);
	}

	/* Set current node as target node */
	XML_SetUserData(parser,list);
		
}

/****************************************************************************

        End Element

****************************************************************************/
void ElementEnd(void *userData,const XML_Char *name)
{
	llist_t * list = (llist_t *)userData;

	XML_SetUserData(parser,get_parent(list));
}

/****************************************************************************

        Charcter data handler

****************************************************************************/

void CharacterHandler(void *userData,const XML_Char *s,int len)
{
	llist_t * list = (llist_t *)userData;
	int i = 0;

	/* skip first useless bytes */
	while( len > 0 && ( s[i]=='\t' || s[i]=='\n') ) {
		i++;
		len--;
	}
	/* Remove useless trailing bytes */
	while( len > 0 && ( s[len-1]=='\t' || s[len-1]=='\n') ) {
		len--;
	}

	add_data(list,strndup(s+i,len));
}

/****************************************************************************

        LoadMessData

****************************************************************************/

llist_t *LoadMessData(const char * path)
{
        /* Get MESS data*/
        FILE *fpipe;
        char buffer[1024];
        enum XML_Status status;

        if(path==NULL) {
                return NULL;
        }

        /* Execute mess binary */
        sprintf(buffer,"%s -listxml",path);
        fpipe = (FILE*)popen(buffer,"r");
        if(fpipe==NULL) return NULL;

        /* XML parsing */
        parser=XML_ParserCreate("UTF-8");

        /* Element parsing */
        XML_SetStartElementHandler(parser,ElementStart);
        XML_SetEndElementHandler(parser,ElementEnd);

        /* character data parsing */
        XML_SetCharacterDataHandler(parser,CharacterHandler);

	
        XML_SetUserData(parser,NULL);

        status = XML_STATUS_OK;
        while ( fgets( buffer, sizeof buffer, fpipe) && status!=XML_STATUS_ERROR ) {
                status=XML_Parse(parser,buffer,strlen(buffer),0);
        }
        pclose(fpipe);

        if(status != XML_STATUS_OK) return NULL;

        return root_node;
}
