#include <gtk/gtk.h>
#include <string.h>
#include "parse_data.h"
#include "conf_file.h"
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <zip.h>
#include <ctype.h>
#include <fcntl.h>


gboolean on_SoftwareImage_expose_event(GtkWidget *widget,GdkEventExpose *event,gpointer user_data);

#define NUM_EXT 4
char image_extension[NUM_EXT][9] = {"png","jpg","jpeg","gif"};
static int current_image_path = 0;

/***************************************************************************


  SoftwareShortName 


 ***************************************************************************/
static const char * SoftwareShortName(const char * software)
{
        int i = strlen(software);
        while(i>=0) {
                if(software[i]=='/') {
                        return (software+i+1);
                }
                i--;
        }
        return software;
}


void str_tolower(char * string)
{
	int i;

	for(i=0;i<strlen(string);i++) {
		string[i]=tolower(string[i]);
	}
}

gint on_SoftwareImage_button_release_event(GtkWidget *widget, GdkEventButton *event, gpointer func_data);

static int is_number(char c)
{
	if( c >= '0' && c <= '9' ) return TRUE;
	else return FALSE;
}

/* the returned pointer must be freed */
static char * GetNameWithoutExtension(const char *str)
{
	int i;
	char * name;

	i = strlen(str);
	while( str[i] != '.' && i >= 0)
		i--;

	if(i<0) {
		name=strdup(str);
	}
	else {
		name=malloc(i+1);
		bzero(name,i+1);
		memcpy(name,str,i);
	}
	return (name );
}

static char * GetExtension(const char *str)
{
	int i;

	i = strlen(str);
	while( str[i] != '.' && i >= 0)
		i--;

	if(i<0) return NULL;

	return ((char *)(str+i+1) );
}

static void ReadZip(const char *directory,GtkWidget * SoftwareTree)
{
	GtkListStore * SoftwareListStore;
	GtkTreeIter iter;
	struct zip * ZipFile;
	struct zip_stat ZipStat;
	int i,num_file;
	char tmp[1024];

	SoftwareListStore=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(SoftwareTree)));

	ZipFile = zip_open(directory,0,NULL);
	if( ZipFile == NULL ) return;

	num_file = zip_get_num_files(ZipFile);

	for(i=0;i<num_file;i++) {
		if( zip_stat_index(ZipFile,i,0,&ZipStat) == 0 && ZipStat.size!=0 ) { /*ZipStat.size!=0 is to skip directory entry*/
			sprintf(tmp,"%s/%s",directory,ZipStat.name);
			gtk_list_store_append (SoftwareListStore, &iter);
			gtk_list_store_set (SoftwareListStore, &iter,
					0, SoftwareShortName(ZipStat.name),
					1, tmp,
					2, GetExtension(ZipStat.name),
					-1);

		}
	}

	zip_close( ZipFile );
}
static void ReadDirectory(const char *directory,GtkWidget * SoftwareTree)
{
	GtkTreeIter iter;
	DIR *dp;
	struct dirent *ep;
	GtkListStore * SoftwareListStore;
	struct stat status;
	char tmp[1024];


	SoftwareListStore=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(SoftwareTree)));
	dp = opendir (directory);
	if (dp != NULL)
	{
		while ((ep = readdir (dp)) ) {
			if(strcmp(".",ep->d_name)==0) continue;
			if(strcmp("..",ep->d_name)==0) continue;
			sprintf(tmp,"%s/%s",directory,ep->d_name);
			stat(tmp,&status);
			if(S_ISDIR(status.st_mode)) {
				ReadDirectory(tmp,SoftwareTree);
				continue;
			}
			if(strlen(ep->d_name)>4 && (
				strcmp(ep->d_name+strlen(ep->d_name)-4,".zip")==0 ||
				strcmp(ep->d_name+strlen(ep->d_name)-4,".Zip")==0 ||
				strcmp(ep->d_name+strlen(ep->d_name)-4,".ZIP")==0 )) {
				ReadZip(tmp,SoftwareTree);
				continue;
			}
			gtk_list_store_append (SoftwareListStore, &iter);
			gtk_list_store_set (SoftwareListStore, &iter,
					0, ep->d_name,
					1, tmp,
					2, GetExtension(ep->d_name),
					-1);
		}
		(void) closedir (dp);
	}
}

void GetMachineNameList(const char * machine_name, llist_t * mess, int enable_parent_clones, int * machine_to_parse, char *** machine_array)
{
	llist_t * machine;
	llist_t * parent;
	*machine_to_parse = 0;
	*machine_array = malloc(sizeof(char *));
	if( *machine_array == NULL ) return;
	*machine_to_parse = 1;
	*machine_array[0] = (char *)machine_name;

	if( enable_parent_clones == 1 ) {
		/* Is this machine a parent or a clone */
		machine = find_node_by_attr(mess,"machine","name",machine_name);
		if( machine == NULL ) return;
		if( find_attr(machine,"cloneof") == NULL ) {
			/* this is a parent, seek for clones */
			parent = find_first_node(mess,"machine");
			while ( parent != NULL )  {
				if( find_attr(machine,"cloneof") != NULL && 
						strcmp(find_attr(machine,"cloneof"),machine_name)==0) {
					(*machine_to_parse)++;
					*machine_array = realloc(*machine_array,(*machine_to_parse)*sizeof(char *));
					if( *machine_array == NULL ) {*machine_to_parse = 0;return;}
					(*machine_array)[(*machine_to_parse)-1] = find_attr(machine,"name");
				}
				parent = find_next_node(parent);
			}
		}
		else {
			/* this is a clone, fill the list with clone's name and parent's name */
			*machine_array = realloc(*machine_array,2*sizeof(char *));
			if( *machine_array == NULL ) {*machine_to_parse = 0;return;}
			*machine_to_parse = 2;
			(*machine_array)[1] = find_attr(machine,"cloneof");;
		}

	}
}

void FillSoftwareList(char * machine_name,GtkWidget * SoftwareTree, llist_t * mess, conf_t * conf, int enable_parent_clones)
{
	const char *dummy;
	char *in=NULL;
	char *out=NULL;
	GtkListStore * SoftwareListStore;
	char software_path[1024];
	int machine_to_parse;
	char ** machine_array;
	int i;

	SoftwareListStore=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(SoftwareTree)));
	gtk_list_store_clear(SoftwareListStore);


	/* Read software for parent and clones */
	GetMachineNameList(machine_name, mess, enable_parent_clones, &machine_to_parse, &machine_array);

	for(i=0; i< machine_to_parse; i++ ) {
		/* Get each software path in software_path*/
		out = NULL;
		dummy = conf_get(conf,"mess","software_path");
		if( dummy == NULL ) return;
		in=strdup(dummy);
		if(in == NULL) return;
		out = strtok(in,";");
		while(out!=NULL) {
			/* first try a zip archive with driver name in the root software directory */
			sprintf(software_path,"%s/%s.zip",out,machine_array[i]);
			ReadZip(software_path,SoftwareTree);
			sprintf(software_path,"%s/%s.ZIP",out,machine_array[i]);
			ReadZip(software_path,SoftwareTree);
			sprintf(software_path,"%s/%s.Zip",out,machine_array[i]);
			ReadZip(software_path,SoftwareTree);
			/* Next, try a directory with the driver name */
			/* Construct path */
			sprintf(software_path,"%s/%s",out,machine_array[i]);
			/* list directory and sub-directories content */
			ReadDirectory(software_path,SoftwareTree);
			out = strtok(NULL,";");
		}
	}

	free(machine_array);
}

static void add_image_path(char ***paths,int * paths_num,char * base,const char * machine, const char * software){
	char image_path[1024];
	int j;
	DIR *dp;
	struct dirent *ep;
	char * name;
	int found = 0;
	/* lower case string */
	char * lc_software;
	char * lc_d_name;

	lc_software=strdup(software);
	str_tolower(lc_software);

	name = GetNameWithoutExtension(lc_software);
	if(name==NULL) {
		free(lc_software);
		return;
	}

	/* search for file of type softwareXXXX.ext , or software-XXXX.ext or software_XXXX.ext where X is a number [0-9] */

	sprintf(image_path,"%s/%s",base,machine);

	dp = opendir (image_path);
	if (dp != NULL)
	{
		while ((ep = readdir (dp)) ) {
			if(strcmp(".",ep->d_name)==0) continue;
			if(strcmp("..",ep->d_name)==0) continue;
			lc_d_name = strdup(ep->d_name);
			str_tolower(lc_d_name);
			if(strncmp(name,lc_d_name,strlen(name))==0) { /* the beginning of the filenam is good*/			
				if( (is_number(lc_d_name[strlen(name)]) &&
							is_number(lc_d_name[strlen(name)+1]) &&
							is_number(lc_d_name[strlen(name)+2]) &&
							is_number(lc_d_name[strlen(name)+3]) ) ||

						((lc_d_name[strlen(name)]=='_' || lc_d_name[strlen(name)]=='-') &&
						 is_number(lc_d_name[strlen(name)+1]) &&
						 is_number(lc_d_name[strlen(name)+2]) &&
						 is_number(lc_d_name[strlen(name)+3]) &&
						 is_number(lc_d_name[strlen(name)+4]) ) ) {
					for(j=0;j<NUM_EXT;j++) {
						if(strcmp(GetExtension(lc_d_name),image_extension[j]) == 0) {
							found = 1;
						}
						if(!found) {
							continue;
						}

						/* We have a corresponding image */
						sprintf(image_path,"%s/%s/%s",base,machine,ep->d_name);
						(*paths_num)++;
						*paths = realloc(*paths,(*paths_num)*sizeof(char *));
						(*paths)[(*paths_num)-1] = malloc(strlen(image_path) + 1);
						strcpy((*paths)[(*paths_num)-1],image_path);
						break;
					}
				}
			}
			free(lc_d_name);
		}
	}
	closedir(dp);
	free(name);
}


void ShowSoftwareImage(GtkContainer * container, const char * machine, llist_t * mess, const char * software, conf_t *conf, int enable_parent_clones)
{
        char *in=NULL;
        char *out=NULL;
        char **paths=NULL;
        int paths_number=0;
        const char * dummy;
        GtkWidget *image;
	char * selected_software;
	int machine_to_parse;
	char ** machine_array;
	int i;

	GetMachineNameList(machine, mess, enable_parent_clones, &machine_to_parse, &machine_array);

	for(i=0; i< machine_to_parse; i++ ) {
		/* Get each base path */
		out = NULL;
		dummy = conf_get(conf,"mess","software_snap_path");
		if(dummy==NULL) return;
		in=strdup(dummy);
		if(in == NULL) return;
		out = strtok(in , ";" );
		while(out!=NULL) {
			add_image_path(&paths,&paths_number,out,machine_array[i],software);
			out = strtok(NULL , ";" );
		}
	}

	GtkWidget * previousImage = gtk_bin_get_child(GTK_BIN(container));
	if( previousImage) {
		gtk_widget_destroy(previousImage);
	}

        if(paths_number > 0) {
		if(current_image_path >= paths_number) {
			current_image_path = 0;
		}
		selected_software = paths[current_image_path];
		current_image_path = ( current_image_path + 1 ) % paths_number;
                image = gtk_image_new_from_file(selected_software);
                if(image==NULL) return;


		/*Set the image's size to the window's size */
		GtkAllocation allocation;
		gtk_widget_get_allocation (GTK_WIDGET(container),&allocation);
	

		GdkPixbuf *pSrcPixbuf, *pDestPixbuf;
		gint iWidth, iHeight;

		/* get the GdkPixbuf */
		pSrcPixbuf = gtk_image_get_pixbuf(GTK_IMAGE(image));
		/* Size of the imagea */
		iWidth = gdk_pixbuf_get_width(pSrcPixbuf);
		iHeight = gdk_pixbuf_get_height(pSrcPixbuf);
		float ratio_x = (float)allocation.width / (float)iWidth;
		float ratio_y = (float)allocation.height / (float)iHeight;
		/* Create dest GdkPixbuf */
		if( ratio_x < ratio_y )
			pDestPixbuf = gdk_pixbuf_scale_simple(pSrcPixbuf, iWidth*ratio_x, iHeight*ratio_x, GDK_INTERP_TILES);
		else
			pDestPixbuf = gdk_pixbuf_scale_simple(pSrcPixbuf, iWidth*ratio_y, iHeight*ratio_y, GDK_INTERP_TILES);

		/* Replace old GdkPixbuf by the new one */
		gtk_image_set_from_pixbuf(GTK_IMAGE(image), pDestPixbuf);

		g_object_unref(pDestPixbuf);

		g_signal_connect(image, "expose-event", G_CALLBACK(on_SoftwareImage_expose_event), (gpointer)selected_software);

		/* Add to eventbox to catch event on the image */
		gtk_container_add (GTK_CONTAINER (container), image);

		gtk_widget_show_all (GTK_WIDGET(container));

	}
}

/***************************************************************************


  RenameSnapshot 


 ***************************************************************************/
void RenameSnapshot(char * machine,const char * snap_software,time_t start_time, conf_t *conf)
{
        const char * dummy;
        char *in=NULL;
        char *out=NULL;
	char target_dir[1024];
	char target_soft[1024];
	int index=0;
	DIR *dp;
	struct dirent *ep;
	struct stat status;
	int j;
	int fd_in;
	int fd_out;
	char * buf; /* for copying file */
	int found;

	/* Get first base path */
	dummy = conf_get(conf,"mess","software_snap_path");
	if(dummy == NULL) return;
	out = NULL;
	in=strdup(dummy);
	if(in == NULL) return;
	out = strtok( in , ";" );
	while(out!=NULL) {
		sprintf(target_dir,"%s/%s",out,machine);

		dp = opendir (target_dir);
		if (dp != NULL)
		{
			while ((ep = readdir (dp)) ) {
				if(strcmp(".",ep->d_name)==0) continue;
				if(strcmp("..",ep->d_name)==0) continue;
				sprintf(target_soft,"%s/%s",target_dir,ep->d_name);
				stat(target_soft,&status);
				/* a file has been changed during MESS run */
				if( status.st_mtime > start_time ) {
					/* check it is a known image file */
					found=0;
					for(j=0;j<NUM_EXT;j++) {
						if( strcmp( GetExtension(ep->d_name),image_extension[j])==0) {
							found=1;
						}
					}
					if( !found ) continue; /* not a known image file extension */

					/* try to allocate the requested memory for the copy operation */
					buf = malloc(status.st_size);
					if(buf == NULL) continue;

					/* try to open it */
					fd_in = open(target_soft,O_RDONLY);
					if(fd_in == -1) goto free_memory;

					/* find a suitable filename for the destination */
					char * soft_name_no_ext = GetNameWithoutExtension(snap_software);
					do {
						sprintf(target_soft,"%s/%s-%04d.%s",target_dir,soft_name_no_ext,index,GetExtension(ep->d_name));
						fd_out = open(target_soft,O_CREAT|O_EXCL|O_WRONLY, 
								S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
						index++;
					}
					while( fd_out == -1 && index < 9999);
					free(soft_name_no_ext);
					if(index>9999) goto close_fd_in;

					/* copy the files */
					if ( read(fd_in,buf,status.st_size) != status.st_size) goto close_fd_out;
					if ( write(fd_out,buf,status.st_size) != status.st_size) goto close_fd_out;
close_fd_out:
					close(fd_out);
close_fd_in:
					close(fd_in);
free_memory:
					free(buf);
				}

			}
		}
		(void) closedir (dp);
		out = strtok( NULL , ";" );
	}
}

